import { Controller } from '@hotwired/stimulus';

export default class extends Controller {
  static targets = ['image', 'openIcon'];

  connect() {
    this.isOpen = this.element.dataset.isOpen === 'true';
    this.updateOpenStatusIcon();
    this.parentIds = this.element.dataset.parentIds;
    this.parentId = this.parentIds.split(',').pop();
    this.closeImg = this.element.dataset.closeImg;
    this.openImg = this.element.dataset.openImg;

    this.toggleSubrowsHandler = this.toggleSubrows.bind(this);
    this.closeIfParentClosedHandler = this.closeIfParentClosed.bind(this);
    this.element.addEventListener('click', this.toggleSubrowsHandler);
    this.listContainer = this.element.closest('.toggle-subrows--list');
    this.listContainer.addEventListener('subrows:close', this.closeIfParentClosedHandler);
  }

  disconnect() {
    this.element.removeEventListener('click', this.toggleSubrowsHandler);
    this.listContainer?.removeEventListener('subrows:close', this.closeIfParentClosedHandler);
  }

  toggleSubrows() {
    if (this.isOpen) {
      this.close();
      this.element.dispatchEvent(
        new CustomEvent('subrows:close', {
          bubbles: true,
          detail: { parentId: this.parentId },
        })
      );
    } else {
      this.open();
    }
  }

  updateSubrows() {
    this.subrows = this.listContainer.querySelectorAll(`.toggle-subrows--row[data-parent-id="${this.parentId}"]`);
  }

  close() {
    this.isOpen = false;
    this.updateSubrows();
    this.updateOpenStatusIcon();
    this.subrows.forEach(subrow => {
      subrow.classList.add('hidden');
    });
    this.imageTarget.src = this.closeImg;
  }

  open() {
    this.isOpen = true;
    this.updateOpenStatusIcon();
    this.updateSubrows();
    this.subrows.forEach(subrow => {
      subrow.classList.remove('hidden');
    });
    this.imageTarget.src = this.openImg;
  }

  updateOpenStatusIcon() {
    if (this.isOpen) {
      this.openIconTarget.querySelector('.fa-chevron-down').classList.remove('hidden');
      this.openIconTarget.querySelector('.fa-chevron-right').classList.add('hidden');
    } else {
      this.openIconTarget.querySelector('.fa-chevron-right').classList.remove('hidden');
      this.openIconTarget.querySelector('.fa-chevron-down').classList.add('hidden');
    }
  }

  closeIfParentClosed(event) {
    if (event.detail.parentId !== this.parentId && this.isOpen && this.parentIds.includes(event.detail.parentId)) {
      this.close();
    }
  }
}
