# frozen_string_literal: true

module SpiritHub
  class PhasesController < ApplicationController
    before_action :find_model, only: %i[inline_edit update_inline]

    def inline_edit
      authorize @model

      key = params[:key].to_sym

      render turbo_stream: turbo_stream.update("inline_edit_#{key}_#{@model.id}", partial: 'spirit_hub/phases/inline_edit', locals: { model: @model, key: })
    end

    def update_inline
      authorize @model

      key = inline_update_params[:key].to_sym

      @model.update(key => inline_update_params[key])
      value = find_inline_value(@model, key)

      flash.now[:notice] = 'Valeur mise à jour'
      update_flash_message

      @turbo_actions << turbo_stream.update("inline_value_#{key}_#{@model.id}", partial: 'spirit_hub/common/inline_value', locals: { model: @model, value:, key: })

      render turbo_stream: @turbo_actions
    end

    private

    def find_model
      @model = policy_scope(Phase).find_by!(id: params[:id])
    end

    def inline_update_params
      params.require(:phase).permit(:key, :initial_date, :postponed_date, :actual_date)
    end

    def find_inline_value(model, key)
      model.send(key)&.strftime('%d/%m/%Y') || ''
    end
  end
end
