# frozen_string_literal: true

module SpiritHub
  class TeamActivitiesController < ApplicationController
    before_action :find_models, only: %i[index sort]

    include Datatablable
    include Sectionable

    PER_PAGE = 10
    SAFE_SORTERS = %w[
      to_be_signed_docs
      to_be_signed_for_me_docs
      to_complete_docs
      to_validate_docs
      responses_docs
    ].freeze

    def index
      authorize(User, policy_class:)

      respond_to(&:html)
    end

    def sort
      authorize(User, policy_class:)

      render partial: 'spirit_hub/team_activities/table'
    end

    private

    def default_columns
      {
        first_name: { label: 'Nom', sortable: true },
        to_be_signed_docs: { label: 'À signer', sortable: true, icon: 'fa-file-signature' },
        to_be_signed_for_me_docs: { label: 'À signer pour moi', sortable: true, icon: 'fa-pen' },
        to_complete_docs: { label: 'À compléter', sortable: true, icon: 'fa-pen' },
        to_validate_docs: { label: 'À valider', sortable: true, icon: 'fa-circle-check' },
        responses_docs: { label: 'Réponses', sortable: true, icon: 'fa-share', color: 'blue' }
      }.freeze
    end

    def find_models
      @all_collaborators = policy_scope(User, policy_scope_class:).where(:id.in => current_user.operative_team.operators)
      @all_managers = policy_scope(User, policy_scope_class:).where(:id.in => current_user.operative_team.writers)
      setup_definitions(%i[collaborators managers])
    end
  end
end
