# frozen_string_literal: true

module SpiritHub
  module ApplicationHelper
    def spirit_hub_importmap_tags(entry_point = 'application')
      safe_join [
        javascript_inline_importmap_tag(SpiritHub.configuration.importmap.to_json(resolver: self)),
        javascript_importmap_module_preload_tags(SpiritHub.configuration.importmap),
        javascript_import_module_tag(entry_point)
      ].compact, "\n"
    end

    def format_file_size(bytes)
      return unless bytes

      if bytes >= 1_000_000_000
        "#{(bytes / 1_000_000_000.0).round(2)} Go"
      elsif bytes >= 1_000_000
        "#{(bytes / 1_000_000.0).round(2)} Mo"
      elsif bytes >= 1000
        "#{(bytes / 1000.0).round(2)} Ko"
      else
        "#{bytes} o"
      end
    end

    def find_color_completion(percent)
      return 'spirit-green' if percent == 100

      # return 'spirit-red' if percent < 30

      'spirit-orange'
    end

    def find_icon(model)
      return 'spirit_hub/icons/close_directory.svg' if model.try(:directory?)

      file_extension_icons = {
        'pdf' => 'spirit_hub/icons/doc_pdf.svg',
        'docx' => 'spirit_hub/icons/doc_docx.svg',
        'ppt' => 'spirit_hub/icons/doc_ppt.svg',
        'dwg' => 'spirit_hub/icons/doc_dwg.svg',
        'kls' => 'spirit_hub/icons/doc_kls.svg',
        'image' => 'spirit_hub/icons/doc_image.svg',
        'multiple' => 'spirit_hub/icons/doc_multiple.svg'
      }

      image_extensions = %w[jpg jpeg png gif svg]
      file_ext = model.file_ext

      return file_extension_icons['image'] if image_extensions.include?(file_ext)

      file_extension_icons.fetch(file_ext, 'spirit_hub/icons/doc_undefined.svg')
    end
  end
end
