# frozen_string_literal: true

module SpiritHub
  module Docs
    class RequestSignatureJob < ApplicationJob
      def perform(doc_ids, recipients, current_user_id)
        docs = Doc.where(:id.in => doc_ids)
        recipient_users = User.where(:id.in => recipients)
        current_user = User.find_by!(id: current_user_id)

        envelope = current_user.docusign_envelopes.create!
        envelope.add_docs(docs)
        envelope.add_signers(recipient_users)

        send_via_docusign(envelope)
      end

      private

      def send_via_docusign(envelope)
        envelope_service = ::Signature::Docusign::EnvelopeService.initialize_with_valid_credentials
        envelope_service.create_and_send_docusign_envelope(envelope)
      rescue StandardError => e
        envelope.destroy
        raise e
      end
    end
  end
end
