# frozen_string_literal: true

module SpiritHub
  class OperationPolicy
    attr_reader :user, :operation

    def initialize(user, operation)
      @user = user
      @operation = operation
    end

    def index?
      user.present?
    end

    def sort?
      index?
    end

    def update?
      user.admin? || operation.writers?(user) || operation.responsible?(user)
    end

    def edit?
      update?
    end

    def create?
      user.present?
    end

    def new?
      create?
    end

    def destroy?
      user.admin? || operation.responsible?(user)
    end

    class Scope
      attr_reader :user, :scope

      def initialize(user, scope)
        @user = user
        @scope = scope
      end

      def resolve
        return scope.all if user&.admin?

        scope.readable_by(user)
      end
    end
  end
end
