# frozen_string_literal: true

module SpiritHub
  class DirectoryCompletenessUpdater
    class << self
      def call(*)
        new(*).call
      end
    end

    def initialize(tagset, options = {})
      raise ArgumentError, 'tagset is not a directory' unless tagset&.directory?

      @directory_tagset = tagset
      @options = options
    end

    def call
      directory_tagset.content_completeness = calculate_completeness_percentage
      directory_tagset.current_state = update_current_state

      unless options[:skip_save]
        directory_tagset.save!
        update_parents_completeness
      end

      true
    end

    private

    STATE_COMPLETE = 'Complet'
    STATE_INCOMPLETE = 'À completer'

    attr_reader :directory_tagset, :options

    def calculate_completeness_percentage
      tagset_ids = directory_tagset.all_children_tagsets.map(&:id) + [directory_tagset.id]

      children_completeness = SpiritHub::Completeness.where(:tagset_id.in => tagset_ids).to_a
      return if children_completeness.empty?

      completed_children = children_completeness.select(&:completed).length
      total_children = children_completeness.length

      (completed_children.to_f / total_children * 100).round
    end

    def update_current_state
      return if directory_tagset.content_completeness.nil?

      directory_tagset.content_completeness == 100 ? STATE_COMPLETE : STATE_INCOMPLETE
    end

    def update_parents_completeness
      directory_tagset.parents.select(&:directory?).each { |parent_tagset| self.class.call(parent_tagset, options) }
    end
  end
end
