# frozen_string_literal: true

SpiritHub::Engine.routes.draw do
  scope path: '/spirit_hub', as: 'spirit_hub' do
    resources :favorites, only: %i[index], path: '/' do
      collection do
        post 'sort'
        put 'update_shown_columns'
        put 'update_view_type'
      end

      member do
        delete 'remove'
      end
    end

    resource :to_classify, only: [], controller: 'to_classify' do
      collection do
        get '/', action: :index
        post 'sort'
        put 'update_shown_columns'
        put 'update_view_type'
      end
    end

    resources :todos, only: %i[index] do
      collection do
        post 'sort'
        put 'update_shown_columns'
        put 'update_view_type'
      end
    end

    resources :tagsets, only: %i[] do
      collection do
        get 'search_directories'
        post 'add_doc_form'
        post 'add_doc'
        post 'bulk_delete'
        post 'classify'
        post 'form/:state_key', to: 'tagsets#form', as: 'form'
      end

      member do
        get 'journal_entries'
        get 'doc/:doc_id', to: 'tagsets#doc', as: 'doc'
        get 'docs'
        post 'share'
        post 'respond'
        post 'evaluate'
        post 'request_signature'
        put 'add_favorite'
        put 'remove_favorite'
        delete 'delete_item'
        post 'send_to'
        get 'inline_edit'
        put 'update_inline'
      end
    end

    resources :directories, only: %i[] do
      member do
        get 'index'
        post 'sort'
        put 'update_shown_columns'
        put 'update_view_type'
        delete 'remove/:tagset_id', action: :remove, as: :remove
        post 'bulk_delete'
        post 'add_doc_to_completeness_form/:completeness_id', action: :add_doc_to_completeness_form, as: :add_doc_to_completeness_form
        post 'add_doc_to_completeness/:completeness_id', action: :add_doc_to_completeness, as: :add_doc_to_completeness
        post 'add_doc_to_tagset_doc_form/:tagset_id', action: :add_doc_to_tagset_doc_form, as: :add_doc_to_tagset_doc_form
        post 'add_doc_to_tagset_doc/:tagset_id', action: :add_doc_to_tagset_doc, as: :add_doc_to_tagset_doc
        delete 'remove_doc_from_tagset_doc/:tagset_id/:doc_id', action: :remove_doc_from_tagset_doc, as: :remove_doc_from_tagset_doc
        post 'duplicate'
      end
    end

    resources :uploads, only: %i[show] do
      get '/download/:doc_id', to: 'shares/downloads#download_doc', as: :download
      get '/download_all', to: 'shares/downloads#download_all_docs', as: :download_all
    end

    get 'search', to: 'search#index', as: :search_index
    post 'search/sort', to: 'search#sort'
    get 'search/users', to: 'search#users'
    get 'search/users_or_groups', to: 'search#users_or_groups'
    get 'search/actors', to: 'search#actors'
    get 'search/tagset_docs_as_shared_attachments_to/:tagset_id', to: 'search#tagset_docs_as_shared_attachments'

    scope :shared do
      get '/', to: 'shared#index', as: 'shared_index'
      post 'sort', to: 'shared#sort', as: 'shared_sort'
      put 'update_shown_columns', to: 'shared#update_shown_columns', as: 'shared_update_shown_columns'
      put 'update_view_type', to: 'shared#update_view_type', as: 'shared_update_view_type'
    end

    resources :team_activities, only: %i[index] do
      collection do
        post 'sort'
        put 'update_shown_columns'
        put 'update_view_type'
      end
    end

    namespace :team_activities do
      get :index, to: 'collaborator#index', as: 'collaborator', path: '/:collaborator_id'
      post :sort, to: 'collaborator#sort', as: 'collaborator_sort', path: '/:collaborator_id/sort'
      put :update_shown_columns, to: 'collaborator#update_shown_columns', as: 'collaborator_update_shown_columns', path: '/:collaborator_id/update_shown_columns'
      put :update_view_type, to: 'collaborator#update_view_type', as: 'collaborator_update_view_type', path: '/:collaborator_id/update_view_type'
    end

    resources :operations, only: %i[index] do
      collection do
        post '/', action: :index
        post 'sort'
        post 'new', action: :new, as: :new
        post 'create', action: :create, as: :create
      end

      member do
        post 'edit', action: :edit, as: :edit
        put 'update', action: :update, as: :update
        delete 'destroy', action: :destroy, as: :destroy
      end
    end

    resources :phases, only: %i[] do
      member do
        get 'inline_edit'
        put 'update_inline'
      end
    end

    get 'generate_error', to: 'errors#generate_error'
    post 'generate_error', to: 'errors#generate_error'
    get '*path', to: 'errors#not_found'
  end
end
