# frozen_string_literal: true

require 'factory_bot_rails'
require 'database_cleaner/mongoid'

include FactoryBot::Syntax::Methods # rubocop:disable Style/MixinUsage
DatabaseCleaner.clean

FactoryBot.definition_file_paths << File.join(Rails.root, 'packages', 'spirit_hub', 'spec', 'factories')
FactoryBot.factories.clear
FactoryBot.find_definitions

SpiritHub::HubdocCycle.import_cycle_from_json

def add_doc_tagset(tagset, filename)
  file_path = Rails.root.join("packages/spirit_hub/spec/fixtures/files/#{filename}")
  uploaded_file = ActionDispatch::Http::UploadedFile.new(
    tempfile: File.open(file_path, 'rb'),
    filename: File.basename(file_path)
  )
  tagset.add_doc_to_tagset_doc(uploaded_file, tagset.responsible)
end

password = 'Password1!'
admin = create(:user, roles: ['Administrator'], email: 'admin@test.com', password:)

plan1 = create(:classement_plan, title: 'Plan 1')

operation_carnot = create(:operation, title: 'Carnot', classement_plan_id: plan1.id)
create(:operation, title: 'Maginot', classement_plan_id: plan1.id)

create_list(:user, 10, password:)
create_list(:contact, 10)

terrain = Tagset.where(title: 'Terrain', operation_id: operation_carnot.id).first
terrain ||= create(:tagset_directory, title: 'Terrain', operation_id: operation_carnot.id, responsible: admin.id)
tagset_doc1 = create(:tagset_doc, title: 'Multiple docs')
add_doc_tagset(tagset_doc1, 'pdf_sample.pdf')
add_doc_tagset(tagset_doc1, 'dog.jpg')
tagset_doc1.add_tagset_doc_to_tagset(terrain, admin)
tagset_doc2 = create(:tagset_doc, title: 'Single doc')
add_doc_tagset(tagset_doc2, 'one_page.pdf')
tagset_doc2.add_tagset_doc_to_tagset(terrain, admin)

puts "Rails environment seeded with the following data: #{Rails.env}"
puts "Users: #{User.count}"
puts "Tagsets: #{Tagset.count}"
puts "Completeness: #{SpiritHub::Completeness.count}"
puts "Docs: #{Doc.count}"
