const { defineConfig } = require('cypress');
const webpack = require('@cypress/webpack-preprocessor');
const { addCucumberPreprocessorPlugin } = require('@badeball/cypress-cucumber-preprocessor');

async function setupNodeEvents(on, config) {
  // This is required for the preprocessor to be able to generate JSON reports after each run, and more,
  await addCucumberPreprocessorPlugin(on, config);

  on(
    'file:preprocessor',
    webpack({
      webpackOptions: {
        resolve: {
          extensions: ['.ts', '.js'],
        },
        module: {
          rules: [
            {
              test: /\.feature$/,
              use: [
                {
                  loader: '@badeball/cypress-cucumber-preprocessor/webpack',
                  options: config,
                },
              ],
            },
          ],
        },
      },
    })
  );

  // Make sure to return the config object as it might have been modified by the plugin.
  return config;
}

module.exports = defineConfig({
  e2e: {
    retries: {
      runMode: 2, // Retries in `cypress run`
      openMode: 0, // Retries in `cypress open`
    },
    baseUrl: 'http://localhost:3000',
    defaultCommandTimeout: 10000,
    supportFile: 'cypress/support/index.js',
    specPattern: '**/*.feature',
    viewportWidth: 1920,
    viewportHeight: 1080,
    setupNodeEvents,
    browser: 'chrome',
  },
  downloadsFolder: 'cypress/downloads',
});
