import { Given, When, Then } from '@badeball/cypress-cucumber-preprocessor';
import '../../../support/step_definitions/modal_steps';

Given('There is some directories', () => {
  cy.appFactories([['create', 'tagset_directory', { responsible: '1', title: 'Searchable tagset 1' }]]);
  cy.appFactories([['create', 'tagset_directory', { responsible: '1', title: 'Searchable tagset 2' }]]);
  cy.appFactories([['create_list', 'tagset_directory', 10, { responsible: '1' }]]);
  cy.appFactories([['create', 'tagset_doc', { responsible: '1', tagset_ids: [] }]]);
});

When('I click on the classify button', () => {
  cy.get('#model_table tbody tr:first form[action="/spirit_hub/tagsets/form/classify"] button').click();
});

When('I open the directory {string}', directory => {
  cy.get(`img[alt="${directory}"][data-toggle-subrows-target="image"]`).click();
});

When('I select the directory {string}', directory => {
  cy.get('label').contains(directory).click();
});

When('I submit the form', () => {
  cy.get('button[data-classify-form-target="submitButton"]').click({ force: true });
  cy.waitForTurboResponse();
});

When('I go to the search page', () => {
  cy.get('form[action="/spirit_hub/search"]').submit();
});

Then('I see the share modal', () => {
  cy.get('header h2').contains('Partager');
});

When('I share the doc', () => {
  cy.get('select[name="to[]"]').next('div').click();
  cy.get('.ss-list > div:visible').first().click({ force: true });
  cy.get('input[name="subject"]').first().type('Hey');
  cy.get('textarea[name="message"]').first().type('Hello there!');
  cy.get('button[data-base-form-target="submitButton"]').click();
  cy.waitForTurboResponse();
});

When('I reclassify the doc', () => {
  cy.get('#model_table tbody tr:first td:last div[data-controller="better-dropdown"] button').first().click();
  cy.get('#model_table tbody tr:first td:last div[data-better-dropdown-target="menu"]').contains('Classer').click();
});

Then('I see the existing parent tagset', () => {
  cy.get('#modal').contains('Classement déjà effectué');
});

Given('There is one directory with child directories', () => {
  cy.appFactories([['create', 'tagset_directory', { responsible: '1', title: 'Child Dir 2' }]]).then(r2 => {
    const child_dir2_id = r2[0].id;

    cy.appFactories([['create', 'tagset_directory', { responsible: '1', title: 'Child Dir 1', tagset_ids: [child_dir2_id] }]]).then(r1 => {
      const child_dir1_id = r1[0].id;

      cy.appFactories([['create', 'tagset_doc', { responsible: '1', tagset_ids: [] }]]);
      cy.appFactories([['create', 'tagset_directory', { responsible: '1', title: 'Searchable tagset 1', tagset_ids: [child_dir1_id] }]])
    });
  });
});

Given('There is one directory with one completeness', () => {
  cy.appFactories([['create', 'tagset_doc', { responsible: '1', tagset_ids: [] }]]);

  cy.appFactories([['create', 'tagset_directory', { responsible: '1', title: 'Searchable tagset 1' }]]).then(records => {
    const tagset_id = records[0].id;
    cy.appFactories([['create', 'completeness', { tagset_id: tagset_id, title: 'Completness 1' }]]);
  });
});
