import { Given, When, Then } from '@badeball/cypress-cucumber-preprocessor';

let directory_id;
let tagset_ids = [];
let sub_tagset_ids = [];

Given('There is one directory with other directories and docs', () => {
  cy.appFactories([['create_list', 'tagset_doc', 2, 'with_informed', 'with_file', 'with_state']]).then(records => {
    records[0].forEach(record => {
      tagset_ids.push(record.id);
    });
  });

  cy.appFactories([['create', 'tagset_doc', 'with_state']]).then(records => {
    tagset_ids.push(records[0].id);
  });

  cy.appFactories([['create_list', 'tagset_doc', 2, 'with_informed', 'with_state']]).then(records => {
    records[0].forEach(record => {
      sub_tagset_ids.push(record.id);
    });
  });

  cy.appFactories([['create', 'tagset_directory', { responsible: '1', tagset_ids: sub_tagset_ids }]]).then(records => {
    tagset_ids.push(records[0].id);
  });

  cy.appFactories([['create', 'tagset_directory', 'with_journal_entries', { responsible: '1', id: '1', tagset_ids: tagset_ids }]]).then(records => {
    directory_id = records[0].id;
  });
});

Given('There is a directory with some docs', () => {
  cy.appFactories([['create_list', 'tagset_doc', 5, 'with_state', { responsible: '1' }]]).then(records => {
    records[0].forEach(record => {
      tagset_ids.push(record.id);
    });
  });
  cy.appFactories([['create', 'tagset_directory', 'with_journal_entries', { responsible: '1', id: '1', tagset_ids: tagset_ids }]]).then(records => {
    directory_id = records[0].id;
  });
});

Given('There is a directory with a lot of docs', () => {
  cy.appFactories([['create_list', 'tagset_doc', 20, 'with_state']]).then(records => {
    records[0].forEach(record => {
      tagset_ids.push(record.id);
    });
  });
  cy.appFactories([['create', 'tagset_directory', 'with_journal_entries', { responsible: '1', id: '1', tagset_ids: tagset_ids }]]).then(records => {
    directory_id = records[0].id;
  });
});

Given('There are existing users I want to share a doc with', () => {
  cy.appFactories([['create', 'user', { email: 'john@test.com', first_name: 'John', last_name: 'Doe' }]]);
  cy.appFactories([['create', 'user', { email: 'jack@test.com', first_name: 'Jack', last_name: 'Black' }]]);
  cy.appFactories([['create', 'user', { email: 'josh@test.com', first_name: 'Josh', last_name: 'Homme' }]]);
});

Given('There are existing external contacts I want to share a doc with', () => {
  cy.appFactories([['create', 'contact', { email: 'john@test.com', first_name: 'John', last_name: 'Doe' }]]);
  cy.appFactories([['create', 'contact', { email: 'jack@test.com', first_name: 'Jack', last_name: 'Black' }]]);
  cy.appFactories([['create', 'contact', { email: 'josh@test.com', first_name: 'Josh', last_name: 'Homme' }]]);
});

When('I visit the directory url', () => {
  cy.visit(`/spirit_hub/directories/${directory_id}/index`);
  cy.wait(200);
});

Then('I can share the doc with the existing users', option => {
  cy.get('form[action*="/share_form"] button[type="submit"]').first().click();

  cy.get('select[name="to[]"]').next('div').click();
  cy.contains('.ss-list:visible > .ss-option', 'John Doe (john@test.com)').first().click({ force: true });
  cy.get('select[name="cc[]"]').next('div').click();
  cy.contains('.ss-list:visible > .ss-option', 'Jack Black (jack@test.com)').first().click({ force: true });
  cy.get('select[name="bcc[]"]').next('div').click();
  cy.contains('.ss-list:visible > .ss-option', 'Josh Homme (josh@test.com)').first().click({ force: true });
  cy.get('input[name="subject"]').first().type('Hey');
  cy.get('textarea[name="message"]').first().type('Hello there!');
  cy.get('form[action*="/share"] button[type="submit"]').first().click();

  cy.get('div[id*="informed_badges"]').first().find('img').should('have.length', 3);
  cy.get('div[id*="informed_badges"] img[src*="JD"]').should('exist');
  cy.get('div[id*="informed_badges"] img[src*="JB"]').should('exist');
  cy.get('div[id*="informed_badges"] img[src*="JH"]').should('exist');
});

Then('I can share the doc with the existing user groups', option => {
  cy.get('form[action*="/share_form"] button[type="submit"]').first().click();

  cy.get('select[name="to[]"]').next('div').click();
  cy.contains('.ss-list:visible > .ss-option', 'Groupe: Group One (2 utilisateurs)').first().click({ force: true });
  cy.get('select[name="cc[]"]').next('div').click();
  cy.contains('.ss-list:visible > .ss-option', 'Groupe: Group Two (1 utilisateurs)').first().click({ force: true });
  cy.get('input[name="subject"]').first().type('Hey');
  cy.get('textarea[name="message"]').first().type('Hello there!');
  cy.get('form[action*="/share"] button[type="submit"]').first().click();

  cy.get('div[id*="informed_badges"]').first().find('img').should('have.length', 3);
  cy.get('div[id*="informed_badges"] img[src*="JD"]').should('exist');
  cy.get('div[id*="informed_badges"] img[src*="JB"]').should('exist');
  cy.get('div[id*="informed_badges"] img[src*="JH"]').should('exist');
});

Then('I can share the doc with the external contacts', option => {
  cy.get('form[action*="/share_form"] button[type="submit"]').first().click();

  cy.get('select[name="to[]"]').next('div').click();
  cy.contains('.ss-list:visible > .ss-option', 'John Doe (john@test.com)').first().click({ force: true });
  cy.get('select[name="cc[]"]').next('div').click();
  cy.contains('.ss-list:visible > .ss-option', 'Jack Black (jack@test.com)').first().click({ force: true });
  cy.get('select[name="bcc[]"]').next('div').click();
  cy.contains('.ss-list:visible > .ss-option', 'Josh Homme (josh@test.com)').first().click({ force: true });
  cy.get('input[name="subject"]').first().type('Hey');
  cy.get('textarea[name="message"]').first().type('Hello there!');
  cy.get('form[action*="/share"] button[type="submit"]').first().click();

  cy.get('div[id*="informed_badges"]').first().find('.rounded-full').should('have.length', 3);
  cy.get('div[id*="informed_badges"] .rounded-full').contains('JD').should('exist');
  cy.get('div[id*="informed_badges"] .rounded-full').contains('JB').should('exist');
  cy.get('div[id*="informed_badges"] .rounded-full').contains('JH').should('exist');
});

Then('I can share the doc with new external contacts', option => {
  cy.get('form[action*="/share_form"] button[type="submit"]').first().click();

  cy.get('select[name="to[]"]').next('div').click();
  cy.get('.ss-search:visible input[type="search"]').first().type('invalid@email');
  cy.get('.ss-search:visible .ss-addable').first().click();
  cy.get('.ss-value-text:visible').should('not.exist'); // Does not accept invalid emails

  cy.get('.ss-search:visible input[type="search"]').first().clear();
  cy.get('.ss-search:visible input[type="search"]').first().type('john.doe@gmail.com');
  cy.get('.ss-search:visible .ss-addable').first().click();
  cy.get('.ss-value-text:visible').contains('john.doe@gmail.com').should('exist');

  cy.get('input[name="subject"]').first().type('Hey');
  cy.get('textarea[name="message"]').first().type('Hello there!');
  cy.get('form[action*="/share"] button[type="submit"]').first().click();

  cy.get('div[id*="informed_badges"]').first().find('.rounded-full').should('have.length', 1);
  cy.get('div[id*="informed_badges"] .rounded-full').contains('J').should('exist');
});

Then('I can share the doc with the user groups', option => {
  cy.get('form[action*="/share_form"] button[type="submit"]').first().click();

  cy.get('select[name="to[]"]').next('div').click();
  cy.contains('.ss-list > .ss-option', 'John Doe (john@test.com)').first().click({ force: true });
  cy.get('select[name="cc[]"]').next('div').click();
  cy.contains('.ss-list > .ss-option', 'Jack Black (jack@test.com)').first().click({ force: true });
  cy.get('select[name="to[]"]').next('div').click();
  cy.contains('.ss-list > .ss-option', 'Josh Homme (josh@test.com)').first().click({ force: true });
  cy.get('input[name="subject"]').first().type('Hey');
  cy.get('textarea[name="message"]').first().type('Hello there!');
  cy.get('form[action*="/share"] button[type="submit"]').first().click();

  cy.get('div[id*="informed_badges"]').first().find('img').should('have.length', 3);
  cy.get('div[id*="informed_badges"] img[src*="JD"]').should('exist');
  cy.get('div[id*="informed_badges"] img[src*="JB"]').should('exist');
  cy.get('div[id*="informed_badges"] img[src*="JH"]').should('exist');
});

Then('I should see the right columns', () => {
  cy.get('#model_table td').eq(3).contains('Doc Title 1');
  cy.get('#model_table td').eq(4).contains('7.1 Ko');
  cy.get('#model_table td').eq(5).contains(new Date().toLocaleDateString('fr-FR'));
  cy.get('#model_table td').eq(6).find('div').should('exist');
  cy.get('#model_table td').eq(8).find('img').should('exist');
  cy.get('#model_table td').eq(9).contains('Classé');
  cy.get('#model_table td').eq(10).find('a').should('exist');
});

When('I click on the first directory', () => {
  cy.get('#model_table tbody tr:first td').eq(2).get('div[data-toggle-subrows-target]').should('exist');
  cy.get('[data-controller="toggle-subrows"]').first().click();
});

Then('I should see sub documents', () => {
  cy.get('#model_table tbody tr:visible').should('have.length', 6);
});

Then('I should not see sub documents', () => {
  cy.get('#model_table tbody tr:visible').should('have.length', 4);
});

// Add/Remove doc to completeness
Given('There is a directory with some docs, directories and completness', () => {
  cy.appFactories([['create_list', 'tagset_doc', 3, 'with_state']]).then(records => {
    records[0].forEach(record => {
      tagset_ids.push(record.id);
    });
  });

  cy.appFactories([['create', 'tagset_directory', { responsible: '1' }]]).then(records => {
    tagset_ids.push(records[0].id);
    cy.appFactories([['create', 'completeness', { tagset_id: tagset_ids.at(-1), title: 'Completness subdirectory 1' }]]);
    cy.appFactories([['create', 'completeness', { tagset_id: tagset_ids.at(-1), title: 'Completness subdirectory 2' }]]);
  });

  cy.appFactories([['create', 'tagset_directory', { responsible: '1', id: '1', tagset_ids: tagset_ids }]]).then(records => {
    directory_id = records[0].id;
    cy.appFactories([['create', 'completeness', { tagset_id: directory_id, title: 'Completness directory' }]]);
  });
});

When('I can add a doc to directory completeness', () => {
  cy.get('#model_table tbody tr:visible:last button:last').click();
  cy.get('#modal input[type="file"]').attachFile('files/example.pdf');
  cy.get('form[data-controller="base-form"] button[type="submit"]').click();
  cy.waitForTurboResponse();
  cy.get('#model_table tbody tr:visible').last().should('not.contain', 'À completer');
  cy.get('#tagset_1_global_avancement').contains('33 %');
});

When('I can add a doc to subdirectory completeness', () => {
  cy.get('#model_table tbody tr:visible').eq(-2).find('img:first').click();
  cy.get('#model_table tbody tr:visible').eq(-2).find('button:last').click();
  cy.get('#modal input[type="file"]').attachFile('files/example.pdf');
  cy.get('form[data-controller="base-form"] button[type="submit"]').click();
  cy.waitForTurboResponse();
  cy.get('#model_table tbody tr:visible').eq(-4).should('contain', 'À completer');
  cy.get('#model_table tbody tr:visible').eq(-4).should('contain', '50 %');
  cy.get('#tagset_1_global_avancement').contains('67 %');
});

When('I can remove a doc from directory completeness', () => {
  cy.get('#model_table tbody tr:visible').last().find('button:last').click();
  cy.waitForTurboResponse();
  cy.get('#model_table tbody tr:visible').last().should('contain', 'A compléter');
  cy.get('#tagset_1_global_avancement').contains('33 %');
});

When('I can remove a doc from subdirectory completeness', () => {
  cy.get('#model_table tbody tr:visible').eq(-2).find('button:last').click();
  cy.waitForTurboResponse();
  cy.get('#tagset_1_global_avancement').contains(' 0 %');
  cy.get('#model_table tbody tr:visible').eq(-2).should('contain', 'A compléter');
  cy.get('#model_table tbody tr:visible').eq(-4).should('contain', ' 0 %');
});

When('I can add multiple docs to directory completeness', () => {
  cy.get('#model_table tbody tr:visible:last button:last').click();
  cy.get('#modal input[type="file"]').attachFile(['files/example.pdf', 'files/example.jpeg']);
  cy.get('form[data-controller="base-form"] button[type="submit"]').click();
  cy.waitForTurboResponse();
  cy.get('#model_table tbody tr:visible').eq(-2).should('contain', 'example.pdf');
  cy.get('#model_table tbody tr:visible').eq(-1).should('contain', 'example.jpeg');
});

// Add/Remove doc to tagset_doc
Given('This is some tagset_doc with file', () => {
  cy.appFactories([['create', 'tagset_doc', 'with_file', 'with_state', { responsible: '1' }]]).then(records => {
    tagset_ids.push(records[0].id);
  });

  cy.appFactories([['create', 'tagset_doc', 'with_file', 'with_state', { responsible: '1' }]]).then(records => {
    cy.appFactories([['create', 'tagset_directory', { responsible: '1', tagset_ids: [records[0].id] }]]).then(records => {
      tagset_ids.push(records[0].id);
    });
  });

  cy.appFactories([['create', 'tagset_directory', { responsible: '1', id: '1', tagset_ids: tagset_ids }]]).then(records => {
    directory_id = records[0].id;
  });
});

When('I can add a doc to tagset_doc', () => {
  cy.get('#model_table tbody tr:visible:first button:eq(-2)').click();
  cy.get('#modal input[type="file"]').attachFile('files/example.pdf');
  cy.get('form[data-controller="base-form"] button[type="submit"]').click();
  cy.waitForTurboResponse();
  cy.get('#model_table tbody tr:visible').eq(0).should('contain', 'Doc Title 1');
  cy.get('#model_table tbody tr:visible').eq(1).should('contain', 'dog.jpg');
  cy.get('#model_table tbody tr:visible').eq(2).should('contain', 'example.pdf');
});

When('I can remove a doc to tagset_doc', () => {
  cy.get('#model_table tbody tr:visible').eq(1).find('button:last').click();
});

When('I can add multiple doc to tagset_doc', () => {
  cy.get('#model_table tbody tr:visible:last [data-controller="toggle-subrows"]').first().click();
  cy.get('#model_table tbody tr:visible:last button:eq(-2)').click();
  cy.get('#modal input[type="file"]').attachFile(['files/example.pdf', 'files/example.jpeg']);
  cy.get('form[data-controller="base-form"] button[type="submit"]').click();
  cy.waitForTurboResponse();
  cy.get('#model_table tbody tr:visible').eq(-1).should('contain', 'example.jpeg');
  cy.get('#model_table tbody tr:visible').eq(-2).should('contain', 'example.pdf');
  cy.get('#model_table tbody tr:visible').eq(-3).should('contain', 'dog.jpg');
});

When('I can remove a doc from multiple tagset_doc', () => {
  cy.get('#model_table tbody tr:visible').eq(-2).find('button:last').click();
  cy.waitForTurboResponse();
  cy.get('#model_table tbody tr:visible').eq(-2).should('not.contain', 'example.pdf');
  cy.get('#model_table tbody tr:visible').should('have.length', 5);
  cy.get('#model_table tbody tr:visible').eq(-2).find('button:last').click();
  cy.waitForTurboResponse();
  cy.get('#model_table tbody tr:visible').should('have.length', 3);
});

When('I can remove a tagset_doc with multiple doc', () => {
  cy.get('#model_table tbody tr:visible:first button:eq(-2)').click();
  cy.get('#modal input[type="file"]').attachFile('files/example.pdf');
  cy.get('form[data-controller="base-form"] button[type="submit"]').click();
  cy.waitForTurboResponse();
  cy.get('#model_table tbody tr:visible:first button:last').click();
  cy.get('#model_table tbody tr:visible').should('have.length', 2);
});

Given('There is a doc to validate in a directory', () => {
  cy.appFactories([['create', 'tagset_doc', 'with_state', 'with_informed', { state_key: 'to_validate' }]]).then(records => {
    tagset_ids.push(records[0].id);
  });

  cy.appFactories([['create', 'tagset_directory', { responsible: '1', id: '1', tagset_ids: tagset_ids }]]).then(records => {
    directory_id = records[0].id;
  });
});

Given('There is a additionnal user and group', () => {
  cy.appFactories([['create', 'user', { email: 'johndoe@test.com', first_name: 'John', last_name: 'Doe' }]]);
  cy.appFactories([['create', 'user', { email: 'janedoe@test.com', first_name: 'Jane', last_name: 'Doe' }]]);
  cy.appFactories([['create', 'group', { name: 'Groupe 1' }]]);
  cy.appFactories([['create', 'group', { name: 'Groupe 2' }]]);
});

Then('I can inline edit the responsible', () => {
  cy.get('#inline_value_responsible_1 button').first().click();
  cy.get('#tagset_responsible').next('div').click();
  cy.contains('.ss-list:visible > .ss-option', 'John Doe').first().click();
  cy.get('#inline_value_responsible_1 button').last().click();
  cy.get('#inline_value_responsible_1 div').first().contains('John Doe');
});

Then('I can inline edit the accounted', () => {
  cy.get('#inline_value_accounted_1 button').first().click();
  cy.get('#tagset_accounted').next('div').click();
  cy.contains('.ss-list:visible > .ss-option', 'John Doe').first().click();
  cy.get('#tagset_accounted').next('div').click();
  cy.contains('.ss-list:visible > .ss-option', 'Jane Doe').first().click();
  cy.get('#inline_value_accounted_1 button').last().click();
  cy.get('#inline_value_accounted_1 div').first().contains('John Doe');
  cy.get('#inline_value_accounted_1 div').first().contains('Jane Doe');
});

Then('I can inline edit the start_date', () => {
  cy.get('#inline_value_start_date_1 button').first().click();
  cy.get('#tagset_start_date').type('2021-02-01');
  cy.get('#inline_value_start_date_1 button').last().click();
  cy.get('#inline_value_start_date_1 div').first().contains('01/02/2021');
});

Then('I can inline edit the end_date', () => {
  cy.get('#inline_value_end_date_1 button').first().click();
  cy.get('#tagset_end_date').type('2021-02-01');
  cy.get('#inline_value_end_date_1 button').last().click();
  cy.get('#inline_value_end_date_1 div').first().contains('01/02/2021');
});

Given('There a directory with an operation with phases', () => {
  cy.appFactories([['create', 'operation', { title: 'Operation 1' }]]).then(records => {
    const operation_id = records[0].id;
    cy.appFactories([['create', 'phase', { title: 'Phase 1', operation_id: operation_id }]]);
    cy.appFactories([['create', 'phase', { title: 'Phase 2', operation_id: operation_id }]]);
    cy.appFactories([['create', 'phase', { title: 'Phase 3', operation_id: operation_id }]]).then(records => {
      const phase_id = records[0].id;
      cy.appFactories([['create', 'tagset_directory', { responsible: '1', id: '1', phase_id: phase_id, operation_id: operation_id }]]).then(records => {
        directory_id = records[0].id;
        cy.appFactories([['create', 'completeness', { title: 'Completeness 1', tagset_id: directory_id }]]);
      });
    });
  });
});

When('I open phase 1', () => {
  cy.get('button[data-action="accordion#toggle"]').contains('Phases du projet').first().click();
  cy.get('header[data-action="click->accordion#toggle"]').contains('Phase 1').first().click();
});

Then('I can inline edit {string}', date_string => {
  cy.get(`div[id*=inline_value_${date_string}] button`).first().click();
  cy.get(`#phase_${date_string}`).type('2021-02-01');
  cy.get(`#phase_${date_string}`).next('div').find('button[type="submit"]').click();
  cy.get(`div[id*=inline_value_${date_string}] div`).first().contains('01/02/2021');
});

Then('The percent of the phase should be updated', () => {
  cy.get('#model_table tbody tr:visible:last button:last').click();
  cy.get('#modal input[type="file"]').attachFile('files/example.pdf');
  cy.get('form[data-controller="base-form"] button[type="submit"]').click();
  cy.waitForTurboResponse();
  cy.get('button[data-action="accordion#toggle"]').contains('Phases du projet').first().click();
  cy.get('h3').contains('Phase 3').next('div').contains('100 %');
});

Given('There is a directory with directory that can be duplicated', () => {
  cy.appFactories([['create', 'tagset_directory', { responsible: '1', max_duplicates: '1' }]]).then(records => {
    tagset_ids.push(records[0].id);
  });

  cy.appFactories([['create', 'tagset_directory', { responsible: '1', id: '1', tagset_ids: tagset_ids }]]).then(records => {
    directory_id = records[0].id;
  });
});

Then('I can duplicate', () => {
  cy.get('#model_table tbody tr:visible:first button[title="Dupliquer"]').click();
  cy.get('#model_table tbody tr:visible').should('have.length', 2);
  cy.get('#model_table tbody tr:first td').eq(3).contains('Tagset Title 1');
  cy.get('#model_table tbody tr:first td').eq(2).get('div[data-toggle-subrows-target]').should('not.exist');;
  cy.get('#model_table tbody tr:last td:last button').should('have.length', 1);
});
