import { Given, When, Then } from '@badeball/cypress-cucumber-preprocessor';
import '../../../support/step_definitions/doc_view_steps';

let upload_id;
let doc_ids;

Given('There is a shared doc', () => {
  cy.appFactories([['create', 'tagset_doc', 'with_file', { filename: 'pdf_sample.pdf' }]]).then(records => {
    doc_ids = records[0].doc_ids;

    cy.appFactories([['create', 'upload', { user_id: '1', doc_ids: doc_ids }]]).then(records => {
      upload_id = records[0].id;
    });
  });
});

Given('There are multiple shared docs', () => {
  cy.appFactories([['create', 'tagset_doc', 'with_files', { filenames: ['pdf_sample.pdf', 'pdf_sample2.pdf'] }]]).then(records => {
    doc_ids = records[0].doc_ids;

    cy.appFactories([['create', 'upload', { user_id: '1', doc_ids: doc_ids }]]).then(records => {
      upload_id = records[0].id;
    });
  });
});

When('I visit the shared doc page', () => {
  cy.visit(`/spirit_hub/uploads/${upload_id}`);
});

Then('I can see the list of shared docs', () => {
  cy.get('#docs-download-list').contains('pdf_sample.pdf').should('exist');
  cy.get('#docs-download-list').contains('pdf_sample2.pdf').should('exist');
});

Then('I can download each of them separately', () => {
  cy.intercept('GET', `**/download/${doc_ids[0]}`).as('fileDownload_1');
  cy.intercept('GET', `**/download/${doc_ids[1]}`).as('fileDownload_2');

  cy.get('#docs-download-list').contains('pdf_sample.pdf').prev('a[download="true"]').click();
  cy.wait('@fileDownload_1').then(interception => {
    expect(interception.response.statusCode).to.eq(200);
  });

  cy.get('#docs-download-list').contains('pdf_sample2.pdf').prev('a[download="true"]').click();
  cy.wait('@fileDownload_2').then(interception => {
    expect(interception.response.statusCode).to.eq(200);
  });
});

Then('I can download all of them together', () => {
  cy.intercept('GET', '**/download_all').as('fileDownload');

  cy.get('#docs-download-list').contains('span', 'Télécharger tout (2)').parents('a[download="true"]').first().click();
  cy.wait('@fileDownload').then(interception => {
    expect(interception.response.statusCode).to.eq(200);
  });
});
