import { Given, When, Then } from '@badeball/cypress-cucumber-preprocessor';
import 'cypress-file-upload';

Given('I am guest', () => {
  cy.app('clean');
});

Given('I am a logged in admin', () => {
  cy.app('clean');
  cy.appFactories([
    [
      'create',
      'user',
      {
        id: '1',
        email: 'admin@test.com',
        password: 'Password1!',
        roles: ['Administrator'],
        authentication_token: 'johns_token',
        first_name: 'Admin',
        last_name: 'User',
      },
    ],
  ]);
  cy.addAuthHeader('johns_token');
});

When('I visit the url {word}', url => {
  let headers = {};
  if (Cypress.env('auth-user-token')) {
    headers = {
      'X-auth-user-token': Cypress.env('auth-user-token'),
    };
  }
  cy.visit({
    url: `${Cypress.config().baseUrl}${url}`,
    failOnStatusCode: false,
    headers,
  });
  cy.wait(200);
});

Then('The url should match {string}', urlPattern => {
  const dynamicUrl = urlPattern.replace(/\*/g, '.*');
  const regex = new RegExp(`^${Cypress.config().baseUrl}${dynamicUrl}$`);
  cy.url().should('match', regex);
});

Then('The url is {word}', url => {
  cy.url().should('eq', `${Cypress.config().baseUrl}${url}`);
});

When('I select {string} per page', text => {
  cy.get('select[name="per_page"]').select(text);
});

When('I remove the title column', () => {
  cy.get('#column-selector button').click();
  cy.wait(100);
  cy.get('#column-selector label[for="columns_cleaned_main_key"]').click();
});

When('I readd the title column', () => {
  cy.get('#column-selector label[for="columns_cleaned_main_key"]').click();
});

Then('I see {string} columns', n => {
  cy.get('#model_table tbody tr:first td').should('have.length', n);
});

When('I click on the grid view button', () => {
  cy.get('#view-type-selector button:first').click();
});

When('I click on the table view button', () => {
  cy.get('#view-type-selector button:last').click();
});

Then('I see the grid view', () => {
  cy.get('#model_table .grid').should('exist');
});

Then('I see the table view', () => {
  cy.get('#model_table table').should('exist');
});

// Sort
Then('I can sort', () => {
  cy.get('#model_table td').eq(3).contains('Doc Title 1');
  cy.get('#model_table th').eq(3).find('button').click();
  cy.get('#model_table td').eq(3).contains('Doc Title 5');
});

// Per page and pagination
Then('I can paginate', () => {
  cy.get('select[name="per_page"]').select('5');
  cy.get('#model_table tbody tr').should('have.length', 6);
  cy.get('.pagination a').eq(0).should('have.class', 'cursor-not-allowed');
  cy.get('.pagination a.bg-primary').contains('1');
  cy.get('.pagination a').eq(4).contains('4');
  cy.get('.pagination a').contains('3').click();
  cy.get('#model_table tbody tr').should('have.length', 6);
  cy.get('.pagination a').eq(0).should('not.have.class', 'cursor-not-allowed');
  cy.get('.pagination a.bg-primary').contains('3');
  cy.get('.pagination a').eq(4).contains('4');
});

// Columns selector
Then('I can select all {int} columns', count => {
  cy.get('#model_table tbody tr:first td').should('have.length', count);
  cy.get('#column-selector button').click();
  cy.wait(200);
  cy.get('#column-selector label[for="columns_cleaned_main_key"]').click();
  cy.get('#model_table tbody tr:first td').should('have.length', count - 1);
  cy.get('#column-selector label[for="columns_cleaned_main_key"]').click();
  cy.get('#model_table tbody tr:first td').should('have.length', count);
});

// View selector
Then('I can switch between views', () => {
  cy.get('#model_table table').should('exist');
  cy.get('#view-type-selector button:first').click();
  cy.get('#model_table .grid').should('exist');
  cy.get('#view-type-selector button:last').click();
  cy.get('#model_table table').should('exist');
});

//  Add remove from favorites
Then('I can add and remove from favorites {string}', option => {
  cy.get('#model_table td').eq(0).find('button').as('favoriteButton');
  cy.get('#model_table td').eq(0).find('svg').as('favoriteIcon');

  cy.get('@favoriteButton').click();
  cy.get('@favoriteIcon').should('have.class', 'text-yellow-500');

  cy.get('@favoriteButton').click();
  cy.get('@favoriteIcon').should('have.class', 'text-gray-200');

  if (option === 'with grid') {
    cy.get('#view-type-selector button:first').click();
    cy.get('.card form').filter('[action*="/add_favorite"]').first().as('addFavoriteForm');
    cy.get('@addFavoriteForm').submit();
    cy.get('.card').eq(0).find('svg').should('have.class', 'text-yellow-500');
    cy.get('.card form').filter('[action*="/remove_favorite"]').first().as('removeFavoriteForm');
    cy.get('@removeFavoriteForm').submit();
    cy.get('.card').eq(0).find('svg').should('have.class', 'text-gray-200');
  }
});

// Remove
Then('I can remove element {string}', option => {
  cy.get('#model_table tbody tr:first .remove-button').click();
  cy.get('#model_table tbody tr').should('have.length', 5);
  if (option === 'with grid') {
    cy.get('#view-type-selector button:first').click();
    cy.get('.grid .remove-button').first().click({ force: true });
    cy.get('#view-type-selector button:last').click();
    cy.get('#model_table tbody tr').should('have.length', 4);
  }
});

// Bulk delete
Then('I can bulk delete {string}', option => {
  cy.get('#model_table tbody tr').eq(0).find('td:eq(1) input').check();
  cy.get('#model_table tbody tr').eq(1).find('td:eq(1) input').check();
  cy.get('#model_table button[formaction="/spirit_hub/tagsets/bulk_delete"]').click();
  cy.get('#model_table tbody tr').should('have.length', 4);
  if (option === 'with grid') {
    cy.get('#view-type-selector button:first').click();
    cy.get('.grid>div:first input:first').check({ force: true });
    cy.get('.grid>div:eq(2) input:first').check({ force: true });
    cy.get('#model_table button[formaction="/spirit_hub/tagsets/bulk_delete"]').click();
    cy.get('#view-type-selector button:last').click();
    cy.get('.grid>div').should('have.length', 1);
  }
});

Then('I can share {string}', option => {
  cy.get('div[id*="informed_badges"]')
    .first()
    .then($div => {
      const before_img_length = $div.find('img').length;

      cy.get('#model_table tbody tr:first form').filter('[action="/spirit_hub/tagsets/form/share"]').first().submit();
      cy.get('select[name="to[]"]').next('div').click();
      cy.get('.ss-list > div:visible').first().click({ force: true });
      cy.get('input[name="subject"]').first().type('Hey');
      cy.get('textarea[name="message"]').first().type('Hello there!');
      cy.get('form[action*="/share"]').first().submit();
      cy.get('div[id*="informed_badges"]')
        .first()
        .find('img')
        .should('have.length', Math.min(4, before_img_length + 1));
      cy.get('div[id*="last_current_user_name"]').first().contains('Admin User');
    });

  if (option === 'with grid') {
    cy.get('#view-type-selector button:first').click();

    cy.get('div[id*="informed_badges"]')
      .eq(1)
      .then($div => {
        const before_img_length = $div.find('img').length;

        cy.get('.grid form[action="/spirit_hub/tagsets/form/share"]').eq(1).submit();
        cy.get('select[name="to[]"]').next('div').click();
        cy.get('.ss-list > div:visible').first().click({ force: true });
        cy.get('input[name="subject"]').first().type('Hey');
        cy.get('textarea[name="message"]').first().type('Hello there!');
        cy.get('form[action*="/share"]').first().submit();
        cy.get('div[id*="informed_badges"]')
          .eq(1)
          .find('img')
          .should('have.length', Math.min(4, before_img_length + 1));
      });
  }
});

Then('I can classify {string}', option => {
  cy.get('#model_table tbody tr:first td:last div[data-controller="better-dropdown"] button').first().click();
  cy.get('#model_table tbody tr:first td:last div[data-better-dropdown-target="menu"]').contains('Classer').click();
  cy.get('#directory_selector label').first().click();
  cy.get('button[data-classify-form-target="submitButton"]').click();
  cy.waitForTurboResponse();
  cy.get('#model_table tbody tr:first').contains('Classé');

  if (option === 'with grid') {
  }
});

Then('I can make tagset done {string}', option => {
  cy.get('#model_table tbody tr:first td:last div[data-controller="better-dropdown"] button').first().click();
  cy.get('#model_table tbody tr:first td:last div[data-better-dropdown-target="menu"]').contains('Fin de traitement').click();
  cy.waitForTurboResponse();
  cy.get('#model_table tbody tr:first').contains('Traité');

  if (option === 'with grid') {
  }
});

Then('I can validate {string}', option => {
  cy.get('#model_table tbody tr:first form[action="/spirit_hub/tagsets/form/validate"] button').click();
  cy.get('textarea[name="message"]').type('This is a message');
  cy.get('select[name="recipients[]"]').next('div').click();
  cy.contains('.ss-list:visible > .ss-option', 'Admin User (admin@test.com)').first().click({ force: true });
  cy.get('button[data-base-form-target="submitButton"]').click({ force: true });
  cy.waitForTurboResponse();
  cy.get('#model_table tbody tr:first').contains('Validé');

  if (option === 'with grid') {
  }
});

Then('I can respond {string}', option => {
  cy.get('#model_table tbody tr:first td:last div[data-controller="better-dropdown"] button').first().click();
  cy.get('#model_table tbody tr:first td:last div[data-better-dropdown-target="menu"]').contains('Répondre').click();
  cy.get('select[name="to[]"]').next('div').click();
  cy.get('.ss-list > div:visible').first().click({ force: true });
  cy.get('input[name="subject"]').first().type('Hey');
  cy.get('textarea[name="message"]').first().type('Hello there!');
  cy.get('form[action*="/respond"]').first().submit();
  if (option === 'with grid') {
  }
});

When('I add a doc without directory', () => {
  cy.get('form[action="/spirit_hub/tagsets/add_doc_form"]').submit();
  cy.get('input[data-classify-form-target="fileInput"]').attachFile('files/example.pdf');
  cy.get('form[action="/spirit_hub/tagsets/add_doc"]').submit();
  cy.waitForTurboResponse();
});

When('I add a doc with a directory', () => {
  cy.get('form[action="/spirit_hub/tagsets/add_doc_form"]').submit();
  cy.get('input[data-classify-form-target="fileInput"]').attachFile('files/example.pdf');
  cy.get('input[data-classify-form-target="tagsetRadio"]').first().check();
  cy.get('form[action="/spirit_hub/tagsets/add_doc"]').submit();
  cy.waitForTurboResponse();
});
