# frozen_string_literal: true

require_relative '../../app/helpers/spirit_hub/application_helper'

module SpiritHub
  class Engine < ::Rails::Engine
    isolate_namespace SpiritHub

    config.autoload_paths << root.join('lib')
    config.autoload_paths << root.join('formatters')

    initializer 'spirit_hub.importmap', before: 'importmap' do |app|
      app.config.importmap.cache_sweepers << Engine.root.join('app/assets/javascripts')
      app.config.importmap.paths << Engine.root.join('config/initializers/importmap.rb')

      ActiveSupport.on_load(:action_controller_base) do
        before_action do
          SpiritHub.configuration.draw_importmap if app.config.importmap.sweep_cache && Rails.application.importmap.cache_sweeper&.updated?
        end
      end
    end

    initializer 'spirit_hub.assets.precompile' do |app|
      app.config.assets.paths << root.join('vendor', 'javascript')
      app.config.assets.paths << root.join('vendor', 'stylesheet')

      app.config.assets.precompile += Dir[SpiritHub::Engine.root.join('app', 'assets', 'images', '**', '*.{png,jpg,jpeg,gif}')].map do |path|
        Pathname.new(path).relative_path_from(SpiritHub::Engine.root.join('app', 'assets')).to_s
      end
    end

    initializer 'spirit_hub.load_overrides' do
      Rails.application.config.to_prepare do
        overrides_path = Engine.root.join('app', 'overrides', '**', '*.rb')

        Dir.glob(overrides_path).each do |override_file|
          load override_file
        end
      end
    end
  end
end
