# frozen_string_literal: true

require 'faker'

FactoryBot.define do
  factory :operation, class: 'SpiritHub::Operation' do
    sequence(:code) { |n| "#{Faker::Code.asin}-#{n}" }
    title { Faker::Company.name }
    city  { Faker::Address.city }
    address { Faker::Address.street_address }
    postal_code { Faker::Address.zip_code }
    agency { Faker::Company.name }
    type { Faker::Company.industry }
    start_date { Faker::Date.between(from: 5.days.ago, to: 1.days.ago) }
    end_date { Faker::Date.between(from: Date.today, to: 5.days.from_now) }

    transient do
      current_user { nil }
      responsible_user { nil }
    end

    current_user_id { current_user&.id || create(:user).id }
    responsible { responsible_user || create(:user).id }

    trait :with_tagset_root do
      after(:create) do |operation|
        tagset_root = create(:tagset_directory, operation:, title: "Projet #{operation.title}", responsible: operation.responsible)
        operation.set(root_tagset_id: tagset_root.id)
      end
    end
  end
end
