# frozen_string_literal: true

require 'rails_helper'

RSpec.describe(SpiritHub::Completeness, type: :model) do
  it 'has a valid factory' do
    expect(true).to be_truthy
  end

  describe 'update_tagset_completeness' do
    let!(:directory) { create(:tagset_directory) }
    let!(:other_directory) { create(:tagset_directory) }
    let!(:completeness1) { create(:completeness, tagset: directory, min_objects: 1, object_ids: [1]) }
    let!(:completeness2) { create(:completeness, tagset: directory, min_objects: 2, object_ids: [1, 2]) }
    let!(:completeness3) { create(:completeness, tagset: directory, min_objects: 0, object_ids: []) }
    let!(:other_completeness) { create(:completeness, tagset: other_directory, min_objects: 5, object_ids: []) }

    it 'updates the tagset completeness' do
      expect(directory.reload.content_completeness).to eq(100)
    end

    it 'when removing an object' do
      completeness1.update(object_ids: [])
      expect(directory.reload.content_completeness).to eq(67)
    end

    it 'when 0 min objects' do
      [completeness1, completeness2, completeness3].each do |completeness|
        completeness.update(min_objects: 0)
      end

      expect(directory.reload.content_completeness).to eq(100)
    end

    it 'when removing completeness' do
      completeness1.update(object_ids: [])
      expect(directory.reload.content_completeness).to eq(67)

      [completeness1, completeness2, completeness3].each(&:destroy)

      expect(directory.reload.content_completeness).to be_nil
    end

    it 'when changing tagset' do
      completeness1.update(object_ids: [])
      expect(directory.reload.content_completeness).to eq(67)

      new_directory = create(:tagset_directory)
      completeness1.update(tagset: new_directory)
      expect(directory.reload.content_completeness).to eq(100)
      expect(new_directory.reload.content_completeness).to eq(0)
    end
  end

  describe 'udpate_completed' do
    let!(:tagset) { create(:tagset) }
    let!(:completeness) { create(:completeness, tagset:) }

    it 'default' do
      expect(completeness.min_objects).to eq(1)
      expect(completeness.object_ids).to eq([])
      expect(completeness.completed).to be_falsey
    end

    it 'when object_ids changed' do
      completeness.update(min_objects: 1)
      expect(completeness.completed).to be_falsey

      completeness.update(object_ids: [1])
      expect(completeness.completed).to be_truthy

      completeness.update(object_ids: [])
      expect(completeness.completed).to be_falsey

      completeness.update(object_ids: [1, 2, 3])
      expect(completeness.completed).to be_truthy

      completeness.update(object_ids: nil)
      expect(completeness.completed).to be_falsey
    end
  end
end
