# frozen_string_literal: true

require 'rails_helper'

RSpec.describe SpiritHub::SharedPolicy do
  let(:user) { create(:user) }

  subject { described_class }

  permissions :index?, :sort? do
    it 'permits all users' do
      expect(subject).to permit(user)
    end
  end

  describe SpiritHub::SharedPolicy::Scope do
    let!(:tagset_in) { create(:tagset_doc, informed: ['1'], readers: [user.id.to_s]) }
    let!(:not_in_all_can_access) { create(:tagset_doc, informed: ['1']) }
    let!(:not_informed) { create(:tagset_doc, readers: [user.id.to_s]) }
    let!(:scope) { Tagset.all }

    subject { described_class.new(user, scope).resolve }

    it 'returns the right tagset' do
      expect(subject).to match_array([tagset_in])
    end
  end
end
