# frozen_string_literal: true

require 'rails_helper'

RSpec.describe SpiritHub::ToClassifyPolicy do
  let(:user) { create(:user) }

  subject { described_class }

  permissions :index?, :sort? do
    it 'permits all users' do
      expect(subject).to permit(user)
    end
  end

  describe SpiritHub::ToClassifyPolicy::Scope do
    let!(:tagset_in) { create(:tagset_doc, responsible: user.id) }
    let!(:tagset_with_parent) { create(:tagset_doc, responsible: user.id) }
    let!(:parent_tagset) { create(:tagset_directory, tagset_ids: [tagset_with_parent.id]) }
    let!(:not_responsible_tagset) { create(:tagset_doc, responsible: nil) }
    let!(:scope) { Tagset.all }

    subject { described_class.new(user, scope).resolve }

    it 'returns the right tagset' do
      expect(subject).to match_array([tagset_in])
    end
  end
end
