# frozen_string_literal: true

require 'rails_helper'

RSpec.describe SpiritHub::TodosPolicy do
  let(:user) { create(:user) }

  subject { described_class }

  permissions :index?, :sort? do
    it 'permits all users' do
      expect(subject).to permit(user)
    end
  end

  describe SpiritHub::TodosPolicy::Scope do
    let!(:tagset_in) { create(:tagset_doc, percent: 80, accounted: [user.id.to_s]) }
    let!(:tagset_without_parent) { create(:tagset_doc, percent: 80, accounted: [user.id.to_s]) }
    let!(:complete_tagset) { create(:tagset_doc, percent: 100, accounted: [user.id.to_s]) }
    let!(:not_in_spirit_tagset) { create(:tagset_doc, spirit_hubdoc: false, percent: 10, accounted: [user.id.to_s]) }
    let!(:parent_tagset) { create(:tagset_directory, tagset_ids: [tagset_in.id, complete_tagset.id, not_in_spirit_tagset.id]) }
    let!(:scope) { Tagset.all }

    subject { described_class.new(user, scope).resolve }

    it 'returns the right tagset' do
      expect(subject).to match_array([tagset_in])
    end
  end
end
